==> welltested-1.0.0.tar.gz: lib/src/models/test_request_data.dart <==
/*
 * Created on Tue Mar 07 2023
 *
 * Copyright (C) 2023 PixelApps - All Rights Reserved
 * The code in this file is Intellectutal Property of PixelApps
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:convert';
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:welltested/src/models/variable.dart';

class TestRequestData {
  final AstNode fileNode;
  final String functionName;
  final DartType classType;
  final Uri fileImportPath;
  final Map<String, MethodDeclaration> methodAstNodes;
  final List<String> simpleModels;
  final List<ClassElement> complexModels;
  final List<String> nestedMethods;
  final List<List<SimpleIdentifier>> callFlows;
  final List<Variable> variableDeclarations;

  const TestRequestData({
    required this.nestedMethods,
    required this.fileNode,
    required this.functionName,
    required this.methodAstNodes,
    required this.classType,
    required this.fileImportPath,
    required this.simpleModels,
    required this.complexModels,
    required this.callFlows,
    required this.variableDeclarations,
  });
  String get className => classType.getDisplayString(withNullability: false);

  String get fileCode => fileNode.toSource();
  String get codeInScrutity {
    String code = fileCode;
    for (final method in methodAstNodes.entries) {
      if (method.key != functionName && !nestedMethods.contains(method.key)) {
        code =
            code.replaceAll(methodAstNodes[method.key]?.toSource() ?? '', '');
      } else if (method.key == functionName) {
        String methodCode = methodAstNodes[method.key]!.toSource();
        for (var i = 0; i < variableDeclarations.length; i++) {
          final Variable variable = variableDeclarations[i];
          if (!methodCode.contains(variable.variableWithType)) {
            methodCode = methodCode.replaceFirst(
                variable.name, variable.variableWithType);
          }
        }
        code =
            code.replaceAll(methodAstNodes[method.key]!.toSource(), methodCode);
      }
    }
    return code;
  }

  List<String> get testcases {
    final method = methodAstNodes[functionName];
    List<String> casesFromAnnotation = [];
    if (method != null) {
      method.sortedCommentAndAnnotations.forEach((e) {
        if (e is Annotation && e.name.name == "Testcases") {
          final testcasesArgument = e.arguments?.arguments.first as ListLiteral;
          if (testcasesArgument.elements.isNotEmpty) {
            casesFromAnnotation = testcasesArgument.elements
                .map((element) => (element as StringLiteral).stringValue ?? "")
                .toList();
          }
        }
      });
    }

    return casesFromAnnotation;
  }

  List<String> get flowPointers {
    List<String> pointers = [];
    callFlows.forEach((flow) {
      if (flow.length > 1) {
        for (int i = 1; i < flow.length; i++) {
          final parameterField = flow[i].staticElement?.name;
          final parameterType =
              flow[i].staticType?.getDisplayString(withNullability: true);
          final parameterParent =
              flow[i - 1].staticType?.getDisplayString(withNullability: false);
          pointers
              .add("$parameterParent().$parameterField return $parameterType");
        }
      }
    });
    return pointers;
  }

  Map<String, dynamic> toJson() => {
        "function_name": functionName,
        "class_name": className,
        "code": codeInScrutity,
        "file_import_path": fileImportPath.toString(),
        "models": jsonEncode(simpleModels),
        "complex_models": jsonEncode(complexModels.map((e) => e.name).toList()),
        "flow_pointers": jsonEncode(flowPointers),
        "nested_methods": jsonEncode(nestedMethods),
        "testcases": jsonEncode(testcases)
      };
}

==> welltested-1.0.5.tar.gz: lib/src/models/test_request_data.dart <==
/*
 * Created on May 1st 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:convert';

import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:welltested/src/models/variable.dart';

class TestRequestData {
  final AstNode fileNode;
  final ClassDeclaration classNode;
  final String functionName;
  final DartType classType;
  final String fileImportPath;
  final Map<String, MethodDeclaration> methodAstNodes;
  final List<String> simpleModels;
  final List<ClassElement> complexModels;
  final List<String> nestedMethods;
  final List<List<SimpleIdentifier>> callFlows;
  final List<Variable> variableDeclarations;
  bool forceGenerate;

  TestRequestData({
    required this.nestedMethods,
    required this.classNode,
    required this.fileNode,
    required this.functionName,
    required this.methodAstNodes,
    required this.classType,
    required this.fileImportPath,
    required this.simpleModels,
    required this.complexModels,
    required this.callFlows,
    required this.variableDeclarations,
    this.forceGenerate = false,
  });
  String get className => classType.getDisplayString(withNullability: false);

  String get fileCode => fileNode.toSource();
  String get codeInScrutity {
    String code = fileCode;
    for (final method in methodAstNodes.entries) {
      if (method.key != functionName && !nestedMethods.contains(method.key)) {
        code =
            code.replaceAll(methodAstNodes[method.key]?.toSource() ?? '', '');
      } else if (method.key == functionName) {
        String methodCode = methodAstNodes[method.key]!.toSource();
        for (var i = 0; i < variableDeclarations.length; i++) {
          final Variable variable = variableDeclarations[i];
          if (!methodCode.contains(variable.variableWithType)) {
            methodCode = methodCode.replaceFirst(
                variable.name, variable.variableWithType);
          }
        }
        code =
            code.replaceAll(methodAstNodes[method.key]!.toSource(), methodCode);
      }
      // remove annotation from the code
      final annotationString = classNode.metadata
          .where((element) => element.name.name == "Welltested")
          .first
          .toSource();
      code = code.replaceAll(annotationString, '');
    }
    return code;
  }

  List<String> get testcases {
    final method = methodAstNodes[functionName];
    List<String> casesFromAnnotation = [];
    if (method != null) {
      method.sortedCommentAndAnnotations.forEach((e) {
        if (e is Annotation && e.name.name == "Testcases") {
          final testcasesArgument = e.arguments?.arguments.first as ListLiteral;
          if (testcasesArgument.elements.isNotEmpty) {
            casesFromAnnotation = testcasesArgument.elements
                .map((element) => (element as StringLiteral).stringValue ?? "")
                .toList();
          }
        }
      });
    }

    return casesFromAnnotation;
  }

  List<String> get flowPointers {
    List<String> pointers = [];
    callFlows.forEach((flow) {
      if (flow.length > 1) {
        for (int i = 1; i < flow.length; i++) {
          final parameterField = flow[i].staticElement?.name;
          final parameterType =
              flow[i].staticType?.getDisplayString(withNullability: true);
          final parameterParent =
              flow[i - 1].staticType?.getDisplayString(withNullability: false);
          pointers
              .add("$parameterParent().$parameterField return $parameterType");
        }
      }
    });
    return pointers;
  }

  Map<String, dynamic> toJson() => {
        "function_name": functionName,
        "class_name": className,
        "code": codeInScrutity,
        "file_import_path": fileImportPath,
        "models": jsonEncode(simpleModels),
        "complex_models": jsonEncode(complexModels.map((e) => e.name).toList()),
        "flow_pointers": jsonEncode(flowPointers),
        "nested_methods": jsonEncode(nestedMethods),
        "testcases": jsonEncode(testcases),
        "force_generate": jsonEncode(forceGenerate),
      };
}
